<?php    

include(dirname(__FILE__) . '/log.php');


class payway extends PaymentModule  //this declares the class and specifies it will extend the standard payment module
{
    
    function __construct()
    {
        $this->name = 'payway';
        $this->tab = 'payments_gateways';
        $this->version = 1;

        parent::__construct(); // The parent construct is required for translations

        $this->page = basename(__FILE__, '.php');
        $this->displayName = $this->l('PayWay Net');
        $this->description = $this->l('Process Payments using the PayWay system');

  }


  public function install( )
  {
    if (!parent::install()
          OR !$this->registerHook('payment')
          OR !$this->registerHook('paymentReturn')
          OR !$this->registerHook('shoppingCartExtra')
          OR !$this->registerHook('backBeforePayment')
          OR !$this->registerHook('paymentReturn')
          OR !$this->registerHook('rightColumn')
          OR !$this->registerHook('cancelProduct')
          OR !$this->registerHook('adminOrder'))
          return false;



    Db::getInstance()->Execute('INSERT INTO '._DB_PREFIX_.'order_state (
      `invoice`, `color`, `unremovable`) VALUES (0, "#FFDD99", 1)');


    $curStatesResults = Db::getInstance()->getRow( 'SELECT MAX(id_order_state) as count from '._DB_PREFIX_.'order_state' );
    $curStatesAfter = $curStatesResults['count'];


    Configuration::updateValue( 'PAYWAY_AWAITING_CONFIRMATION', $curStatesAfter );


    Db::getInstance()->Execute('INSERT INTO '._DB_PREFIX_.'order_state_lang (
      `id_order_state`, `id_lang`, `name`, `template`)
      VALUES ('.(int) $curStatesAfter . ', 1, "Waiting for PayWay Server Confirmation",
      "preparation")');


    return true;


  }

  public function uninstall()
  {
    /* Delete all configurations */
    Configuration::deleteByName( 'PAYWAY_PROXY_HOST' );
    Configuration::deleteByName( 'PAYWAY_PROXY_PORT' );
    Configuration::deleteByName( 'PAYWAY_PROXY_USER' );
    Configuration::deleteByName( 'PAYWAY_PROXY_PASSWORD' );
    Configuration::deleteByName( 'PAYWAY_USERNAME' );
    Configuration::deleteByName( 'PAYWAY_PASSWORD' );
    Configuration::deleteByName( 'PAYWAY_KEY' );
    Configuration::deleteByName( 'PAYWAY_BILLERCODE' );
    Configuration::deleteByName( 'PAYWAY_MERCHANT' );
    Configuration::deleteByName( 'PAYWAY_BASEURL' );
    Configuration::deleteByName( 'PAYWAY_AWAITING_CONFIRMATION' );
    Configuration::deleteByName( 'PAYWAY_PAYPALEMAIL' );
    Configuration::deleteByName( 'PAYWAY_LOGDIRECTORY' );

    return parent::uninstall();
  }

  /**
  * hookPayment($params)
  * Called in Front Office at Payment Screen - displays user this module as payment option and performs token request
  */
  public function hookPayment($params)
  {
    if ( !$this->active)
    {
        debugLog( 'Not Active, returning' );
        return;
    }
    
    debugLog( 'Beginning PayWay Payment for cart: ' . $cart->id );

    global $cookie, $smarty, $cart, $order;

    global $username;
    global $password;
    global $billerCode;
    global $merchantId;


    $smarty->assign(array(
    'this_path' => $this->_path,
    'this_path_ssl' => (Configuration::get('PS_SSL_ENABLED') ? 'https://' : 'http://').htmlspecialchars($_SERVER['HTTP_HOST'], ENT_COMPAT, 'UTF-8').__PS_BASE_URI__.'modules/'.$this->name.'/'
    ));


    //build parameter array for token request
    $params = array();
    $params['username'] = Configuration::get( 'PAYWAY_USERNAME' );
    $params['password'] = Configuration::get( 'PAYWAY_PASSWORD' );
    $params['biller_code'] = Configuration::get( 'PAYWAY_BILLERCODE' );
    $params['merchant_id'] = Configuration::get( 'PAYWAY_MERCHANT' );
    $params['paypal_email'] = Configuration::get( 'PAYWAY_PAYPALEMAIL' );
    $params['payment_reference'] = $cart->id; //get the order ID to use as reference
    $params['payment_amount'] = $cart->getOrderTotal( true, Cart::BOTH );
    $params['payment_amount_change'] = 'false';
    $params['payment_reference_change'] = 'false';

    $token = $this->getToken( $params );

    if ( $token == NULL || strlen($token) == 0 )
    {
        debugLog( 'Error generating token, PayWay option removed' );
        
        return $this->display(__FILE__, 'payment_redirect_error.tpl' );
    }

    else
    {
        $smarty->assign( array( 'paywayToken' => $token,
                                'paymentURL' => Configuration::get( 'PAYWAY_BASEURL' ) . 'MakePayment',
                                'billerCode' => $params['biller_code'] ) );

        debugLog( 'Redirecting Customer : ' . Configuration::get( 'PAYWAY_BASEURL' )
                    . 'MakePayment?token=' . $token . '&biller_code=' . $params['biller_code']);

        //initiate payway payment - this will cause a redirect.
        return $this->display(__FILE__, 'payment_redirect.tpl' );
    }
  }


  //function will request and return the token
  private function getToken( $parameters )
    {
      if ( Configuration::get('PAYWAY_PROXY_HOST') )
      {
        $proxyhost = Configuration::get('PAYWAY_PROXY_HOST');
        $proxyport = Configuration::get('PAYWAY_PROXY_PORT');

        if ( Configuration::get('PAYWAY_PROXY_USER'))
          $proxyuser = Configuration::get('PAYWAY_PROXY_USER');
        else
          $proxyuser = null;

        if ( Configuration::get('PAYWAY_PROXY_PASSWORD') )
          $proxypass =  Configuration::get('PAYWAY_PROXY_PASSWORD');
        else
          $proxypass = null;
      }

      else
      {
        $proxyhost = null;
        $proxyport = null;
        $proxyuser = null;
        $proxypass = null;
      }


        // Find the port setting, if any.
        $payWayUrl = Configuration::get( 'PAYWAY_BASEURL' );
        $port = 443;
        $portPos = strpos( Configuration::get( 'PAYWAY_BASEURL' ), ":", 6 );
        $urlEndPos = strpos( Configuration::get( 'PAYWAY_BASEURL' ), "/", 8 );
        if ( $portPos !== false && $portPos < $urlEndPos )
        {
            $port = (int)substr( Configuration::get( 'PAYWAY_BASEURL' ), ((int)$portPos) + 1, ((int)$urlEndPos));
            $payWayUrl = substr( Configuration::get( 'PAYWAY_BASEURL' ), 0, ((int)$portPos))
                . substr( Configuration::get( 'PAYWAY_BASEURL' ), ((int)$urlEndPos), strlen(Configuration::get( 'PAYWAY_BASEURL' )));
            debugLog( "Found alternate port setting: " . $port );
        }


        $ch = curl_init( Configuration::get( 'PAYWAY_BASEURL' ) . "RequestToken" );


        debugLog( 'Configuring token request to : ' . $payWayUrl . 'RequestToken' );

        if ( $port != 443 )
        {
            curl_setopt( $ch, CURLOPT_PORT, $port );
            debugLog( "Set port to: " . $port );
        }

        curl_setopt( $ch, CURLOPT_FAILONERROR, true );
        curl_setopt( $ch, CURLOPT_FORBID_REUSE, true );
        curl_setopt( $ch, CURLOPT_FRESH_CONNECT, true );
        curl_setopt( $ch, CURLOPT_RETURNTRANSFER, true );

        // Set proxy information as required
        if ( !is_null( $proxyhost ) && !is_null( $proxyport ) )
        {

            curl_setopt( $ch, CURLOPT_HTTPPROXYTUNNEL, true );
            curl_setopt( $ch, CURLOPT_PROXY, $proxyhost . ":" . $proxyport );

            if ( !is_null( $proxyuser) )
            {
                if ( is_null( $proxypass ) )
                {
                    curl_setopt( $ch, CURLOPT_PROXYUSERPWD, $proxyuser . ":" );
                }
                else
                {
                    curl_setopt( $ch, CURLOPT_PROXYUSERPWD,
                        $proxyuser . ":" . $proxypass );
                }
            }
        }

        // Set timeout options
        curl_setopt( $ch, CURLOPT_CONNECTTIMEOUT, 30 );
        curl_setopt( $ch, CURLOPT_TIMEOUT, 30 );

        debugLog( 'Loading cacerts from : ' . dirname(__FILE__) . DIRECTORY_SEPARATOR . 'cacerts.crt');
        // Set references to certificate files
        curl_setopt( $ch, CURLOPT_CAINFO, dirname(__FILE__) . DIRECTORY_SEPARATOR . 'cacerts.crt' );

        // Check the existence of a common name in the SSL peer's certificate
        // and also verify that it matches the hostname provided
        curl_setopt( $ch, CURLOPT_SSL_VERIFYHOST, 1 );

        // Verify the certificate of the SSL peer
        curl_setopt( $ch, CURLOPT_SSL_VERIFYPEER, true );


        // Build the parameters string to pass to PayWay
        $parametersString = '';
        $logParameterString = '';
        $init = true;
        foreach ( $parameters as $paramName => $paramValue )
        {
            if ( $init )
            {
                $init = false;
            }
            else
            {
                $parametersString = $parametersString . '&';
                $logParameterString = $logParameterString . '&';
            }

            $parametersString = $parametersString . urlencode($paramName) . '=' . urlencode($paramValue);

            if ( $paramName != 'password' )
            {
                $logParameterString = $logParameterString . $paramName . '=' . $paramValue;
            }

            else
            {
                $logParameterString = $logParameterString . $paramName . '=' . '********';
            }
        }

        curl_setopt( $ch, CURLOPT_POSTFIELDS, $parametersString );

        debugLog( "Token Request POST: " . $logParameterString );


        // Make the request
        $responseText = curl_exec($ch);

        debugLog( "Token Response: " . $responseText );

        // Check the response for errors
        $errorNumber = curl_errno( $ch );
        if ( $errorNumber != 0 )
        {
            trigger_error( "CURL Error getting token: Error Number: " . $errorNumber .
                ", Description: '" . curl_error( $ch ) . "'" );
            debugLog( "Errors: " . curl_error( $ch ) );

        }

        curl_close( $ch );

        // Split the response into parameters
        $responseParameterArray = explode( "&", $responseText );
        $responseParameters = array();
        foreach ( $responseParameterArray as $responseParameter )
        {
            list( $paramName, $paramValue ) = explode( "=", $responseParameter, 2 );
            $responseParameters[ $paramName ] = $paramValue;
        }

        if ( array_key_exists( 'error', $responseParameters ) )
        {
            trigger_error( "Error getting token: " . $responseParameters['error'] );
        }
        else
        {
            debugLog( 'Token Received : ' . $responseParameters['token']);
            return $responseParameters['token'];
        }
  }


  public function getContent()
  {
    $this->_html .= '<h2>'.$this->l('PayWay').'</h2>';

    $this->_postProcess();

    $this->_setPayWaySubscription();

    $this->_setConfigurationForm();

    return $this->_html;
  }


  private function _setConfigurationForm()
  {
    $this->_html .= '
    <form method="post" action="'.htmlentities($_SERVER['REQUEST_URI']).'">
      <script type="text/javascript">
        var pos_select = '.(($tab = (int)Tools::getValue('tabs')) ? $tab : '0').';
      </script>
      <script type="text/javascript" src="'._PS_BASE_URL_._PS_JS_DIR_.'tabpane.js"></script>
      <link type="text/css" rel="stylesheet" href="'._PS_BASE_URL_._PS_CSS_DIR_.'tabpane.css" />
      <input type="hidden" name="tabs" id="tabs" value="0" />
      <div class="tab-pane" id="tab-pane-1" style="width:100%;">
         <div class="tab-page" id="step2">
          <h4 class="tab">'.$this->l('Settings').'</h2>
          '.$this->_getSettingsTabHtml().'
        </div>
      </div>
      <div class="clear"></div>
      <script type="text/javascript">
        function loadTab(id){}
        setupAllTabs();
      </script>
    </form>';
  }

  //Draw form for user setting import
  private function _getSettingsTabHtml()
  {

    //Work out value we want to display, if configuration item is currently empty, provide the default here too (for ease of output later)

    $user = Configuration::get( 'PAYWAY_USERNAME' );
    $pass = Configuration::get( 'PAYWAY_PASSWORD' );
    $key = Configuration::get( 'PAYWAY_KEY' );
    $biller = Configuration::get( 'PAYWAY_BILLERCODE' );
    $proxyhost = Configuration::get( 'PAYWAY_PROXY_HOST' );
    $proxyport = Configuration::get( 'PAYWAY_PROXY_PORT' );
    $proxyuser = Configuration::get( 'PAYWAY_PROXY_USER' );
    $proxypass = Configuration::get( 'PAYWAY_PROXY_PASSWORD' );

    //The following parameters have default values
    $merchant = Configuration::get( 'PAYWAY_MERCHANT' ) ? Configuration::get( 'PAYWAY_MERCHANT' ) : 'TEST';
    $paypalemail = Configuration::get( 'PAYWAY_PAYPALEMAIL' ) ? Configuration::get( 'PAYWAY_PAYPALEMAIL' ) : 'test@example.com';

    $url = Configuration::get( 'PAYWAY_BASEURL' ) ? Configuration::get( 'PAYWAY_BASEURL' ) : 'https://www.payway.com.au/';

    $loggingDirectory = Configuration::get( 'PAYWAY_LOGDIRECTORY' );



    $html = '
    <h2>'.$this->l('Settings').'</h2>
    <label>'.$this->l('Username:').'</label>
    <div class="margin-form" style="padding-top:2px;">
      <input type="text" name="payway_username" value="' . $user . '" size="30" />
    </div>
    <label>'.$this->l('Password:').'</label>
    <div class="margin-form" style="padding-top:2px;">
      <input type="text" name="payway_password" value="'. $pass . '" size="30" />
    </div>
    <label>'.$this->l('Encryption Key:').'</label>
    <div class="margin-form" style="padding-top:2px;">
      <input type="text" name="payway_key" value="'. $key . '" size="30" />
    </div>
    <label>'.$this->l('Biller Code:').'</label>
    <div class="margin-form" style="padding-top:2px;">
      <input type="text" name="payway_billercode" value="'. $biller . '" size="30" />
    </div>
    <label>'.$this->l('Merchant Id:').'</label>
    <div class="margin-form" style="padding-top:2px;">
      <input type="text" name="payway_merchant" value="'. $merchant . '" size="30" />
    </div>
    <p>Leaving the \'Logging Directory\' blank will disable logging.</p>
    <label>'.$this->l('Logging Directory:').'</label>
    <div class="margin-form" style="padding-top:2px;">
      <input type="text" name="payway_logDir" value="'. $loggingDirectory . '" size="30" />
    </div>

    <p>If you do not use the paypal functionality leave this field blank</p>
    <label>'.$this->l('Paypal Email:').'</label>
    <div class="margin-form" style="padding-top:2px;">
      <input type="text" name="payway_paypalemail" value="'. $paypalemail . '" size="30" />
    </div>

    <h2>The following settings only apply if you use a proxy</h2>
    <label>'.$this->l('ProxyHost:').'</label>
    <div class="margin-form" style="padding-top:2px;">
      <input type="text" name="payway_proxyhost" value="'. $proxyhost . '" size="30" />
    </div>
    <label>'.$this->l('ProxyPort:').'</label>
    <div class="margin-form" style="padding-top:2px;">
      <input type="text" name="payway_proxyport" value="'. $proxyport . '" size="30" />
    </div>
    <label>'.$this->l('Proxy Username:').'</label>
    <div class="margin-form" style="padding-top:2px;">
      <input type="text" name="payway_proxyuser" value="'. $proxyuser . '" size="30" />
    </div>
    <label>'.$this->l('Proxy Password:').'</label>
    <div class="margin-form" style="padding-top:2px;">
      <input type="text" name="payway_proxypassword" value="'. $proxypass . '" size="30" />
    </div>



    <p><strong>Do not change the following URL without consulting the PayWay technical support team</strong></p><br />
    <label>'.$this->l('PayWay Base URL').'</label>
    <div class="margin-form" style="padding-top:2px;">
      <input type="text" name="payway_url" value="'. $url . '" size="30" />
    </div>


    <p class="center"><input class="button" type="submit" name="submitPayWay" value="'.$this->l('Save settings').'" /></p>';
    return $html;
  }

  //process the submitted settings form
  private function _postProcess()
  {
    $this->_errors = NULL;

    if (Tools::isSubmit('submitPayWay'))
    {
      $template_available = array('A', 'B', 'C');
      if (Tools::getValue('payway_username') == NULL )
        $this->_errors[] = $this->l( 'Invalid Username' );
      if (Tools::getValue('payway_password') == NULL )
        $this->_errors[] = $this->l( 'Invalid Password' );
      if (Tools::getValue('payway_key') == NULL )
        $this->_errors[] = $this->l( 'Invalid Encryption Key' );
      if (Tools::getValue('payway_billercode') == NULL )
        $this->_errors[] = $this->l( 'Invalid Biller Code' );
      if (Tools::getValue('payway_merchant') == NULL )
        $this->_errors[] = $this->l( 'Invalid Merchant ID' );
      if (Tools::getValue('payway_url') == NULL )
        $this->_errors[] = $this->l( 'Invalid URL' );


      if ( Tools::getValue( 'payway_logDir') == NULL )
      {
           Configuration::deleteByName( 'PAYWAY_LOGDIRECTORY' );
      }
      else
      {
          $logDirectory = Tools::getValue( 'payway_logDir' );
          //Identify if we are dealing with an actual location.
          //We should first assume its absolute and then try relative addresses
          if ( !realpath( $logDirectory ) )
          {
              if ( realpath( dirname(__FILE__) . $logDirectory ) )
              {
                  $logDirectory = realpath( dirname(__FILE__) . $logDirectory );
              }
              else if ( realpath( dirname(__FILE__) . '/' . $logDirectory ) )
              {
                  $logDirectory = ( realpath( dirname(__FILE__) . '/' . $logDirectory ) );
              }
              else
              {
                  $this->_errors[] = $this->l( 'Log directory does not exist' );
              }  
          }
      }

      //optional proxy settings
      if ( Tools::getValue( 'payway_proxyhost' ) != NULL )
      {
        //if a proxy host is provided a port must be
        if ( Tools::getValue( 'payway_proxyport' ) != NULL )
        {
          Configuration::updateValue( 'PAYWAY_PROXY_HOST', Tools::getValue( 'payway_proxyhost' ) );
          Configuration::updateValue( 'PAYWAY_PROXY_PORT', Tools::getValue( 'payway_proxyport' ) );


          //optionally set username
          if ( Tools::getValue( 'payway_proxyuser' ) != NULL )
            Configuration::updateValue( 'PAYWAY_PROXY_USER', Tools::getValue( 'payway_proxyuser' ) );
          else
            Configuration::deleteByName( 'PAYWAY_PROXY_USER' );

          //optionally set password
          if ( Tools::getValue( 'payway_proxypassword' ) != NULL )
            Configuration::updateValue( 'PAYWAY_PROXY_PASSWORD', Tools::getValue( 'payway_proxypassword' ) );
          else
            Configuration::deleteByName( 'PAYWAY_PROXY_PASSWORD' );

        }

        else
        {
          $this->_errors[] = $this->l( 'No proxy port provided');
        }

      }

      //if proxy host is not provided, remove all proxy configurations
      else
      {
        Configuration::deleteByName( 'PAYWAY_PROXY_HOST' );
        Configuration::deleteByName( 'PAYWAY_PROXY_PORT' );
        Configuration::deleteByName( 'PAYWAY_PROXY_USER' );
        Configuration::deleteByName( 'PAYWAY_PROXY_PASSWORD' );
      }

      //if there are no errors set the configuration fields
      if ( sizeof( $this->_errors ) == 0 )
      {
        Configuration::updateValue( 'PAYWAY_USERNAME' , Tools::getValue( 'payway_username') );
        Configuration::updateValue( 'PAYWAY_PASSWORD' , Tools::getValue( 'payway_password') );
        Configuration::updateValue( 'PAYWAY_KEY' , Tools::getValue( 'payway_key') );
        Configuration::updateValue( 'PAYWAY_BILLERCODE' , Tools::getValue( 'payway_billercode') );
        Configuration::updateValue( 'PAYWAY_MERCHANT' , Tools::getValue( 'payway_merchant') );
        Configuration::updateValue( 'PAYWAY_BASEURL' , Tools::getValue( 'payway_url' ) );
        
        if (Tools::getValue( 'payway_logDir') != NULL)
        {
            Configuration::updateValue( 'PAYWAY_LOGDIRECTORY', $logDirectory );
        }  

        //paypal email is an optional field - only create config if field exists
        if (Tools::getValue('payway_paypalemail') != NULL )
        {
            Configuration::updateValue( 'PAYWAY_PAYPALEMAIL', Tools::getValue( 'payway_paypalemail' ) );
        }
        else
        {
            Configuration::deleteByName( 'PAYWAY_PAYPALEMAIL' );
        }

        $this->_html = $this->displayConfirmation($this->l('Settings updated'));

      }


      //if there are errors generate output
      else
      {
        $error_msg = '<br />';
        foreach ($this->_errors AS $error)
          $error_msg .= $error.'<br />';
        $this->_html = $this->displayError($error_msg);
      }
    }
  }

  private function _setPayWaySubscription()
  {
    $this->_html .= '

    <b>'.$this->l('This module allows you to accept payments by PayWay.').'</b><br /><br />
    '.$this->l('If your client selects this payment mode they will directed to the payway site').'<br />
    '.$this->l('Upon payment completion they will be returned to Prestashop.').'
    '.$this->l('Please configure the settings before using this module.').'
    <div style="clear:both;">&nbsp;</div>';
  }

}
?>