<?php
if (!function_exists('getallheaders'))
{
    function getallheaders()
    {
        foreach ($_SERVER as $name => $value)
        {
            if (substr($name, 0, 5) == 'HTTP_')
            {
                $headers[strtolower(str_replace(' ', '-', ucwords(strtolower(str_replace('_', ' ', substr($name, 5))))))] = $value;
            }
        }
        return $headers;
    }
}

// return true if $str ends with $sub
function endsWith( $str, $sub ) {
  return ( substr( $str, strlen( $str ) - strlen( $sub ) ) == $sub );
}

/*
 * Request a secure token from the PayWay server to protect these payment details.
 *
 * $parameters is the array that maps payment parameter names to parameter values.
 *
 * The return value is the token value that should be included in the hand-off.
 */
function getToken( $parameters )
{
    $proxyHost = strlen( MODULE_PAYMENT_PAYWAY_PROXY_HOST ) == 0 ? null : MODULE_PAYMENT_PAYWAY_PROXY_HOST;
    $proxyPort = strlen( MODULE_PAYMENT_PAYWAY_PROXY_PORT ) == 0 ? null : MODULE_PAYMENT_PAYWAY_PROXY_PORT;
    $proxyUser = strlen( MODULE_PAYMENT_PAYWAY_PROXY_USERNAME ) == 0 ? null : MODULE_PAYMENT_PAYWAY_PROXY_USERNAME;
    $proxyPassword = strlen( MODULE_PAYMENT_PAYWAY_PROXY_PASSWORD ) == 0 ? null : MODULE_PAYMENT_PAYWAY_PROXY_PASSWORD;
    $caCertsFile = dirname(__FILE__) . '/../local/cacerts.crt';
    $payWayUrl = MODULE_PAYMENT_PAYWAY_BASE_URL;
    if ( !endswith( $payWayUrl, "/" ) )
    {
        $payWayUrl = $payWayUrl . "/";
    }
    
    // Find the port setting, if any.
    $port = 443;
    $portPos = strpos( $payWayUrl, ":", 6 );
    $urlEndPos = strpos( $payWayUrl, "/", 8 );
    if ( $portPos !== false && $portPos < $urlEndPos )
    {
        $port = (int)substr( $payWayUrl, ((int)$portPos) + 1, ((int)$urlEndPos));
        $payWayUrl = substr( $payWayBaseUrl, 0, ((int)$portPos))
            . substr( $payWayBaseUrl, ((int)$urlEndPos), strlen($payWayBaseUrl));
    }
    
    $ch = curl_init( $payWayUrl . "RequestToken" );

    if ( $port != 443 )
    {
        curl_setopt( $ch, CURLOPT_PORT, $port );
    }

    curl_setopt( $ch, CURLOPT_FAILONERROR, true );
    curl_setopt( $ch, CURLOPT_FORBID_REUSE, true );
    curl_setopt( $ch, CURLOPT_FRESH_CONNECT, true );
    curl_setopt( $ch, CURLOPT_RETURNTRANSFER, true );

    // Set proxy information as required
    if ( !is_null( $proxyHost ) && !is_null( $proxyPort ) )
    {
        curl_setopt( $ch, CURLOPT_HTTPPROXYTUNNEL, true );
        curl_setopt( $ch, CURLOPT_PROXY, $proxyHost . ":" . $proxyPort );
        if ( !is_null( $proxyUser ) )
        {
            if ( is_null( $proxyPassword ) )
            {
                curl_setopt( $ch, CURLOPT_PROXYUSERPWD, $proxyUser . ":" );
            }
            else
            {
                curl_setopt( $ch, CURLOPT_PROXYUSERPWD, 
                    $proxyUser . ":" . $proxyPassword );
            }
        }
    }

    // Set timeout options
    curl_setopt( $ch, CURLOPT_CONNECTTIMEOUT, 30 );
    curl_setopt( $ch, CURLOPT_TIMEOUT, 30 );

    // Set references to certificate files
    curl_setopt( $ch, CURLOPT_CAINFO, $caCertsFile );

    // Check the existence of a common name in the SSL peer's certificate
    // and also verify that it matches the hostname provided
    curl_setopt( $ch, CURLOPT_SSL_VERIFYHOST, 1 );   

    // Verify the certificate of the SSL peer
    curl_setopt( $ch, CURLOPT_SSL_VERIFYPEER, true );
    
    curl_setopt( $ch, CURLOPT_POSTFIELDS, $parameters );

    
    // Make the request  
    $responseText = curl_exec($ch);
    
    // Check the response for errors
    $errorNumber = curl_errno( $ch );
    if ( $errorNumber != 0 )
    {
        trigger_error( "CURL Error getting token: Error Number: " . $errorNumber . 
            ", Description: '" . curl_error( $ch ) . "'", E_USER_ERROR );
    }

    curl_close( $ch );

    // Split the response into parameters
    $responseParameterArray = explode( "&", $responseText );
    $responseParameters = array();
    foreach ( $responseParameterArray as $responseParameter )
    {
        list( $paramName, $paramValue ) = explode( "=", $responseParameter, 2 );
        $responseParameters[ $paramName ] = $paramValue;
    }
    
    return $responseParameters;
}

?>